#Copyright (c) 2017 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Maksym Kozlov <m.kozlov@3dprinteros.com>

import re

import threaded_sender
from blocking_heating_emulator import BlockingHeatingEmulator


class Sender(threaded_sender.Sender):

    TEMPERATURE_ACKS_PREFIXES = ["T:", "B:"]

    def __init__(self, profile, usb_info):
        threaded_sender.Sender.__init__(self, profile, usb_info)
        self.heating_emulator = BlockingHeatingEmulator(self)

    def define_regexps(self):
        threaded_sender.Sender.define_regexps(self)
        self.temp_re = re.compile('.*ok T:([\d\.]+) /([\d\.]+) .* B:(-?[\d\.]+) /(-?[\d\.]+) .*')
        self.wait_tool_temp_re = re.compile('T:([\d\.]+) /([\d\.]+) @')
        self.wait_platform_temp_re = re.compile('B:([\d\.]+) /([\d\.]+) @')

    def parse_waiting_temperature_updates(self, line):
        match = self.wait_tool_temp_re.match(line)
        if match:
            self.temps[1] = float(match.group(1))
            self.target_temps[1] = float(match.group(2))
            # if self.heating and self.temps[0] >= self.target_temps[0]-3 and self.temps[1] >= self.target_temps[1]-3:
            #     self.heating = False
            #     self.ready_for_command.set()
            self.heating = True

    def parse_waiting_platform_temperature_updates(self, line):
        match = self.wait_platform_temp_re.match(line)
        if match:
            self.temps[0] = float(match.group(1))
            self.target_temps[0] = float(match.group(2))
            # if self.heating and self.temps[0] >= self.target_temps[0]-3 and self.temps[1] >= self.target_temps[1]-3:
            #     self.heating = False
            #     self.ready_for_command.set()
            self.heating = True

    def log_strange_acks(self, line):
        if line[0] == "B" or (len(line) > 1 and line[1] == "B"):
            self.parse_waiting_platform_temperature_updates(line)
            self.operational_flag = True
        else:
            self.logger.warning("Received: " + line.strip())

